# -----------------------------------------------------------------------------
# class ImageSet
#
# An imageset is a cycling set of Pygame images. They must all be the same size.
#
# Copyright 2005 Ron Dippold
# -----------------------------------------------------------------------------

import pygame.sprite, pygame.image
from pygame.locals import *
import random

import Util

class ImageSet:

	def __init__( self, _name, _frames, _center=None, _interval=None ):
		"""
		_name = name for debugging
		_frames = (tuple) of pygame.Surface which must all be the SAME SIZE
		_center = (x,y) of where the center of the image is considered to
			be. If not given, we will figure it out from the _frames[0] size
		_interval = number of seconds between frame flips	
		"""

		# Save the image list, get the first image
		if isinstance( _frames, (list,tuple) ):
			self.frames = _frames
		else:
			self.frames = ( _frames, )

		# if it's only one frame, don't bother deciding which frame to use
		if len( self.frames ) == 1:
			self.newframe = self.current_frame
		else:
			self.newframe = self.real_newframe

		#print "Name:", _name, "frames:", len(self.frames), "center:", _center, "interval:", _interval
		
		# we're starting at the first image
		self.frame = self.frames[0]
		if _center:
			self.center = _center[:]
		else:
			self.center = ( self.frame.get_width()/2, self.frame.get_height()/2)

		self.name = _name
		self.index = 0
		self.time = 0.0
		self.interval = _interval

	# -----------------------------------------
	# Various external uses
	# -----------------------------------------

	def get_center( self ):
		return self.center

	def get_width( self ):
		return self.frame.get_width()

	def get_height( self ):
		return self.frame.get_height()

	def get_first_frame( self ):
		return self.frames[0]

	def set_palette_at( self, _pos, _color ):
		"""
		In all frames, change palette position _pos to _color.
		This only works if we loaded _convert=False
		"""
		for frame in self.frames:
			frame.set_palette_at( _pos, _color )


	# -----------------------------------------
	# Create various NEW transformed ImageSets
	# -----------------------------------------

	def shared_copy( self ):
		"""
		This is basically a shallow copy of the ImageSet. It returns a NEW
		ImageSet with all the frames shared, but separate frame indexes -
		so multiple sprites can use the	same imageset and they won't affect
		which frame is showing for the other ones.
		"""
		return ImageSet( self.name, self.frames, self.center, self.interval )

	def convert( self ):
		"""
		Returns a NEW ImageSet with all the frames convert()ed for speed. You
		would normally only do this if you'd created it with _convert=False.
		But it can also be used to make a deep copy for some reason.
		"""
		frames = [ frame.convert() for frame in self.frames ]

		return ImageSet( self.name, frames, self.center, self.interval )

	def flip( self, _x, _y ):
		"""
		Returns a NEW ImageSet with all the frames flipped around _x and/or
		_y axes, center appropriately adjusted, etc.
		"""
		frames = [ pygame.transform.flip( frame, _x, _y ) for frame in self.frames ]
		center = Util.center_flip( self.center, self.frames[0], _x, _y )

		name = "%s(flip %d,%d)" % (self.name, _x, _y)
		return ImageSet( name, frames, center, self.interval )

	def rotate( self, _angle ):
		"""
		Returns a NEW Imageset with all the frames rotated _angle degrees clockwise.
		"""
		# rotate by -_angle since pygame goes the other way
		frames = [ pygame.transform.rotate( frame, -_angle ) for frame in self.frames ]
		center = Util.center_rotate( self.center, self.frames[0], _angle )

		name = "%s(rot %d)" % (self.name, _angle)
		return ImageSet( name, frames, center, self.interval )

	# -----------------------------------------
	# current_frame just returns the current frame and position,
	# use real_newframe if we have multiple frames.
	# -----------------------------------------
	
	def current_frame( self, _dummy, _pos ):
		"""No images to flip between"""
		rect = self.frame.get_rect()
		rect.topleft = ( _pos[0] - self.center[0], _pos[1] - self.center[1] )
		return self.frame, rect

	def random_frame( self, _pos ):
		"""Choose a random frame"""
		self.index = random.randrange( 0, len(self.frames) )
		self.frame = self.frames[self.index]
		return self.current_frame( 0, _pos )
		
	def real_newframe( self, _seconds, _pos ):
		"""We really have multiple images! Return one."""

		# See if we need to delay between frames
		if self.interval and _seconds:
			self.time += _seconds
			# Not yet, just return the current frame
			if self.time < self.interval:
				return self.current_frame( _seconds, _pos )
			# Yes, go on and flip, also decrement the time till next flip
			else:
				self.time -= self.interval

		# Get next frame
		self.index = (self.index+1) % len(self.frames)
		self.frame = self.frames[self.index]
		
		return self.current_frame( _seconds, _pos )

