# -----------------------------------------------------------------
# Some utility functions that don't exactly fit into any other
# module. Mostly to do with resource loading.
#
# Copyright 2005 Ron Dippold
# -----------------------------------------------------------------

import os
import pygame.image
from   pygame.locals import *

from ImageSet import ImageSet
from MySound import MySound

# -----------------------------------------------------------------
# Set up the loading path and config
# -----------------------------------------------------------------

loader_paths = {}
loader_sprite_config = None
loader_sound_config = None


def loader_config( _paths, _sprite_config, _sound_config ):
	global loader_paths, loader_sprite_config, loader_sound_config
	loader_paths = _paths
	loader_sprite_config = _sprite_config
	loader_sound_config = _sound_config

def get_paths():
	return paths

# -----------------------------------------------------------------
# Load a sound - pretty easy. Uses loader_paths
# -----------------------------------------------------------------
def sound_loader( _base ):

	#fname = os.path.join( loader_paths["sounds"], _base+'.wav' )
	fname = os.path.join( loader_paths["sounds"], _base+'.ogg' )
	# Try to get the user set volume for the sound
	try:
		vol = loader_sound_config.getfloat( 'sound_volume', _base )
	except ConfigParser.NoOptionError:
		vol = 1.0
	return MySound( fname, vol )

# -----------------------------------------------------------------
# This loads an image file or files
# -----------------------------------------------------------------

def imageset_loader( _base, _convert=True, _callback=None ):
	"""
	This loads an image set - if you pass in _base of copter_u,
	it will try to load copter_u.gif, then if that fails copter_u1.gif,
	copter_u2.gif, etc till it runs out of files. All those will be
	turned into one set and must be the same size.

	It will get the center pixel position from _config - the
	'sprites', _base key. If _config is None, it will get the center
	from the height/width.

	It assumes all images are in the loader_config 'sprites' subdirectory and
	that (0,0,0) is transparent.

	If _convert is False, this won't convert. Things draw 6-10x faster when
	converted, so this should only be used with good reason - like your
	routine wants to do something with a paletted color then .convert().

	It will return an ImageSet
	"""

	images = []

	# Try loading just the image
	fname = os.path.join( loader_paths["sprites"], _base+'.gif')
	if os.path.isfile( fname ):
		images += [ pygame.image.load( fname ) ]
		
	# if not, try loading the sequence
	else:
		for i in range(1,10):
			fname = os.path.join( loader_paths["sprites"], _base+`i`+'.gif')
			if os.path.isfile( fname ):
				images += [ pygame.image.load( fname ) ]
			else:
				break

	if not images:
		raise IOError, "No image files found for sprite "+`_base`

	# if _callback, let the caller say exactly what to do with the images
	if _callback:
		images = _callback( images )
	else:
		# almost always set the color keys
		for image in images:
			image.set_colorkey( (0,0,0), RLEACCEL )

		# convert the images for speed
		if _convert:
			images = [ image.convert() for image in images ]
		
	center = get_sprite_center( loader_sprite_config, _base, images[0] )

	framerate = get_sprite_framerate( loader_sprite_config, _base )
	return ImageSet( _base, images, center, framerate )


# NOTE: You don't want to use this any more - use imageset_loader
def image_loader( _files, _convert=True ):
	"""
	This loads an image file or files.
	If you pass it a filename, it will return an image surface.
	If you pass it a tuple of filenames, it will return a tuple of
	image surfaces.

	It assumes all images are in the loader_config 'sprites' subdirectory and
	that (0,0,0) is transparent.

	If _convert is False, this won't convert. Things draw 6-10x faster when
	converted, so this should only be used with good reason - like your
	routine wants to do something with a paletted color then .convert().
	
	"""

	# See if _files is a list or just a standalone. If it's a standalone,
	# make a list, but remember that it was alone

	if isinstance( _files, (tuple,list) ):
		ret_list = True
	elif isinstance( _files, str ):
		ret_list = False
		_files = ( _files, )
	else:
		raise TypeError, "image_loader takes only string or tuple/list of strings"

	# for each file name, load the image and add to list
	images = []
	for f in _files:
		image = pygame.image.load( os.path.join( loader_paths["sprites"],f+'.gif') )
		image.set_colorkey( (0,0,0), RLEACCEL )
		if _convert:
			image = image.convert()
		images.append( image )

	# if we got a list, return a list, otherwise return first item
	if ret_list:
		return images
	else:
		return images[0]


# -----------------------------------------------------------------
# Find a new center for a flipped or rotated image
# -----------------------------------------------------------------
def center_flip( _center, _image, _flipx, _flipy ):
	"""
	Given the _center of _image, figure out where the new center is if
	we flip it over the x and/or y axes
	"""
	cx, cy = _center
	if _flipx:
		cx = _image.get_width() - cx - 1
	if _flipy:
		cy = _image.get_height() - cy - 1
	return (cx, cy)

def center_rotate( _center, _image, _angle ):
	"""
	Given the _center of _image, figure out where the new center is if
	we rotate it _angle degrees. Only 90 supported for now.
	"""
	cx0, cy0 = _center

	# if center was (1,3) and size was 18x18, new center is (14,1)
	if _angle == 90:
		cx = _image.get_height() - cy0 - 1
		cy = cx0
	else:
		raise IndexError, "only _angle of 90 is supported, got "+`_angle`
	return (cx, cy)
	

import ConfigParser
def get_sprite_center( _config, _key, _frame ):
	"""
	Get the center pixel info from the config file.
	Turn 'x, y' into (x,y) - trying to avoid using eval
	"""

	# Get the center setting from the config file, split into x,y
	try:
		txt = _config.get('sprite_center', _key )
	except ConfigParser.NoOptionError:
		txt = "center, center"
	( xtxt, ytxt) = txt.split(',')

	# Figure out what the x center is
	xtxt = xtxt.strip()
	if xtxt=='center':
		x = _frame.get_width()/2
	elif xtxt=='left':
		x = 0
	elif xtxt=='right':
		x = _frame.get_width()-1
	else:
		x = int(xtxt)

	# Figure out what the y center is
	ytxt = ytxt.strip()
	if ytxt=='center':
		y = _frame.get_height()/2
	elif ytxt=='top':
		y = 0
	elif ytxt=='bottom':
		y = _frame.get_height()-1
	else:
		y = int(ytxt)

	# and return it
	return ( x, y )

def get_sprite_framerate( _config, _key ):
	"""
	Get the framerate info from the config file.
	Returns it in fractions of a second.
	"""
	try:
		fps = _config.get('sprite_fps', _key ).strip()
	except ConfigParser.NoOptionError:
		fps = '2'

	if fps=='fast':
		fps = None
	else:
		fps = int(fps)
	if fps:
		fps = 1.0/fps
	return fps



# -----------------------------------------------------------------
# Python has no 'sign()' function! Shame!
# -----------------------------------------------------------------
def sign( _val, _round=None ):
	"""
	Return -1, 0, or 1 depending on whether _val is less than, equal
	to, or greater than 0.  _val is rounded to _round decimal places.
	"""
	# Round it if requested
	if _round:
		_val = round( _val, _round)
	# otherwise a simple if/then
	if _val == 0:
		return 0
	elif _val>0:
		return 1
	else:
		return -1


