# -----------------------------------------------------------------------------
# The big waves at the bottom of the screen.
#
# Copyright 2005 Ron Dippold
# -----------------------------------------------------------------------------



import pygame.sprite, pygame.image
from pygame.locals import *

import math
import Util

# -----------------------------------------------------------------------------
#
# class Wave
#
# -----------------------------------------------------------------------------


class Wave( pygame.sprite.Sprite ):
	"""
	A wave is just a big... wavy... thing a the bottom of the screen that
	moves relentlessly left or right. Two (well, three) of these make up the
	entire width of the screen. They never collide as a sprite with anything
	because the collision matching is too crude.

	Wave takes care of itself mostly - you just need to call Wave.setup()
	to start with, then Wave.static.draw() to draw them once a frame.
	Use Wave.HeightSlope to get heights and slopes for collision.
	"""


	# --------------------------
	# Move the base wave position depending on how much time has passed
	# --------------------------
	@staticmethod
	def __update_master( seconds ):

		# Transition to a new speed, if we are
		if Wave.newspeed:
			Wave.xspeed += Util.sign( Wave.newspeed-Wave.xspeed ) * 20 * seconds
			# If close enough to the new speed, go to the new speed
			if abs( Wave.xspeed - Wave.newspeed) < 5:
				Wave.xspeed = Wave.newspeed
				Wave.newspeed = None
		
		# Move speed pixels/sec * sec
		Wave.xpos += Wave.xspeed * seconds

		# If wave goes off the right side, move it back to the left
		if Wave.xspeed>0 and Wave.xpos >= Wave.size[0]:
			Wave.xpos = 0
		# or if it goes off the left, move it back to the right
		elif Wave.xspeed<0 and Wave.xpos<0:
			Wave.xpos = Wave.size[0]-1

	#
	# Run update on all of our waves
	#
	@staticmethod
	def update_all( seconds ):
		Wave.sprite_list.update( seconds )

	#
	# Draw all the wave sprites, return bounding rect
	#
	@staticmethod
	def draw_all( screen ):
		"""
		Draw all the wave sprites, return bounding rect
		"""
		Wave.sprite_list.draw(screen)
		return [ Wave.rect ]

	newspeed = None
	@staticmethod
	def new_speed( _xspeed ):
		"""
		Transition to a new wave speed.
		"""
		Wave.newspeed = _xspeed

	# ----------------------------
	# Class 'static' initialization
	# ----------------------------

	@staticmethod
	def setup( _smooth, _playrect, _xspeed, _color, _bgcolor ):
		"""
		Wave class static initialization. MUST be called before creating any
		real Wave()s.

		_smooth = use smooth (rounded) waves? If not, angular tops.
		_playrect = play rectangle
		_xspeed = x speed of waves
		_color = ( top_color, bottom_color ) 
		   if _smooth is false only the top_color is used
		_bgcolor = background color
		
		"""

		Wave.size = ( _playrect.width/2, 128 )
		Wave.xpos = 0.0
		Wave.ypos = _playrect.bottom - Wave.size[1]
		Wave.xspeed = _xspeed

		# Now make a wave bitmap
		Wave.__make_image( _smooth, _color, _bgcolor )

		# Create a bounding rectangle for all the waves so we know where to draw
		Wave.rect =	pygame.Rect(0, Wave.ypos, _playrect.width, Wave.size[1] )

		# Make three real Waves - two of these fill the play area - but we need one more for scrolling
		Wave.sprite_list = pygame.sprite.RenderPlain()
		for x in (-1,0,1):
			Wave.sprite_list.add( Wave(x) )

	# ----------------------------
	# Init the 'static' Wave. image
	# ----------------------------

	@staticmethod
	def __make_image( _smooth, _color, _bgcolor ) :
		"""
		Create the Wave image (do this only once)
		"""
		# Create a rectangle, fill it with black
		Wave.image = pygame.Surface( Wave.size ).convert()
		Wave.image.fill( _bgcolor )

		width, height = Wave.size[0], Wave.size[1]
		
		# Also generate a complete array of heights and slopes, for collision detection
		heights = [0,]*width
		Wave.slopes = [0,]*width

		# Figure out how big our wave height top to bottom is
		yheight = height-(height%100)

		if _smooth:
			# Funny enough, smooth is actually easier. Just use cos/sin
			ymid = yheight/2
			for x in xrange(width):
				# figure out x in radians - sin and cos both return -1 to 1
				x_rad = 2 * math.pi * x / width
				heights[x] = int(math.cos(x_rad)*ymid + ymid)

				# map -1 to 1 to -2, -1, 0, 1, 2, with rounding
				# The slope of cos(x) at x is -sin(x)
				Wave.slopes[x] = -int(round(math.sin( x_rad ) * 2))

		else:
			# Do it the classic way - straight line segments
			# Half wave is 8 sections: flat, 1/4 down, 1/2 down, 1/4 half down, flat,
			#	1/4 up, 1/2 up, 1/4 up
			# Total half vertical is n+2n+n=4
			section_slopes = ( 0, 1, 2, 1, 0, -1, -2, -1 )
			xdelta = width/8.0
			ydelta = yheight/4.0

			# Start off at 0,0
			x,y = 0,0
			lastx,lasty = 0,0

			# Add the wave top points
			for i in range(len(section_slopes)):
				# Generate the sparse list, for drawing
				x += xdelta
				y += section_slopes[i]*ydelta

				# Generate the complete height and slope list
				lastslope = (y-lasty)/xdelta
				for x0 in range(lastx,int(x)):
					heights[x0] = int(lasty + (x0-lastx)*lastslope)
					Wave.slopes[x0] = section_slopes[i]
					lastx, lasty = int(x), y

		# figure out color stuff before iterating
		if _smooth:
			rdiff = _color[1][0]-_color[0][0]
			gdiff = _color[1][1]-_color[0][1]
			bdiff = _color[1][2]-_color[0][2]
		else:
			color = _color[0]

		# actually draw the heights
		for x in xrange(width):
			if _smooth:
				# Draw gradiated waves. ooooooo.
				for y in xrange(heights[x],height):
					#col = 80 * (1-(y-heights[x])/float(height))
					#Wave.image.set_at( (x,y), (col,col,175+col) )
					factor = (y-heights[x])/float(height)
					Wave.image.set_at( (x,y), \
									   ( rdiff*factor+_color[0][0],
										 gdiff*factor+_color[0][1],
										 bdiff*factor+_color[0][2] ) )
				Wave.image.set_at( (x,heights[x]), _color[0] )
			else:
				pygame.draw.line( Wave.image, color, (x,heights[x]), (x,height-1) )


		# adjust the height to be relative to the wave's position on the screen
		Wave.top = [ Wave.ypos + height for height in heights ]
		Wave.bottom = Wave.ypos + yheight

		# Generate a HeightSlope helper object to pass around
		Wave.Helper = Wave.HelperObj()


	# --------------------------
	# New Wave sprite - anticlimactic, really!
	# --------------------------
	def __init__( self, _offset=None ):
		"""
		Create a new Wave.
		_offset is offset (in multiples) - -1, 0, or 1
		    or None for a dummy static wave
		"""

		# Our dummy wave, just bail
		if _offset == None:	return
		
		# Always init the sprite
		pygame.sprite.Sprite.__init__( self )

		# We all use the same image
		self.image = Wave.image

		# Set up offset and get a basic rect - we'll set x position in update
		self.offset = _offset
		self.rect = self.image.get_rect()
		self.rect.top = Wave.ypos

		# last time we moved, so we know how far to move next time around


	# --------------------------
	# Move the wave depending on how much time has passed
	# --------------------------
	def update( self, seconds ):
		"""
		All wave positions are just slaves of the master position
		"""

		# Will this always work (in order added? Seems to...)
		if self.offset == -1:
			self.__update_master( seconds )
			
		self.rect.left = self.offset * Wave.size[0] + Wave.xpos

	# -----------------------------------------------------------------------------
	#
	# class Wave.HelperObj
	#
	# -----------------------------------------------------------------------------

	class HelperObj:
		"""
		This is a little helper class for Wave - given a x position it gives you
		the top position or slope of the wave at that point.
		"""

		def get_xspeed( self ):
			"Get the current wave horizontal speed"
			return Wave.xspeed

		def min_top( self ):
			"return the minimum y position to even hit a wave"
			return Wave.ypos

		def max_top( self ):
			"return the maximum y position to even hit a wave"
			return Wave.bottom

		def top( self, _xpos ):
			"return the wave top position at a given x position"

			# Figure out where we are in the height table...
			x0 = int( int(_xpos)-Wave.xpos) % Wave.size[0]

			# And return that plus the y position of the wave base
			#return Wave.ypos + Wave.heights[x0]
			return Wave.top[x0]

		def both( self, _xpos ):
			"Return the wave top position and the slope at a given x position"
			# Figure out where we are in the height table...
			x0 = int( int(_xpos)-Wave.xpos) % Wave.size[0]

			# And return that plus the y position of the wave base
			return Wave.top[x0], Wave.slopes[x0]
		
